# Copyright (C) Fredrick Rybarczyk, 1999. All rights reserved.
# This software is supplied as is without any intent about its
# usefulness for any purpose. It can freely be distributed,
# copied, rewritten and used provided that this message remains
# unchanged. 


###################################################
#                  BASE CLASSES                   #
###################################################
class Tag:
    """Concrete base class for all classes in Formutils."""
    
    def __init__(self, name, attributes=[]):
        """Performs initialisation for all tags"""
        self.rendered = 1
        self.string = "<" + name
        for item in attributes:
            self.string = self.string + " " + item
        self.string = self.string + ">\n"
        
    def __repr__(self):
        self.mk_str()
        return self.string

    def mk_str(self):
        """Override this to create advanced tags as subclass of Tag""" 
        pass
                
        
class Closed_Tag(Tag):
    """Put a list of content within closed tags. Items in the content
    list can be either Tag objects or strings"""  

    def __init__(self, name, attributes=[], content=[]):
        Tag.__init__(self, name, attributes)
        self.rendered = 0 
        self.end_tag  = ""
        self.end_tag  = "</" + name + ">\n"
        self.content  = []
        self.content  = self.content + content
        
    def add_item(self, item):
        self.content.append(item)

    def add_list(self, list):
        self.content = self.content + list

    def mk_str(self):
        """Adds content and closing tag. Tag.mk_str() override."""
        if self.rendered:
            return None 
        for item in self.content:
            try:
                if item.rendered: 
                    # This is necessary for handling content of type
                    # string. 
                    self.string = self.string + item.string
                else:
                    self.string = self.string + str(item)
            except AttributeError:
                self.string = self.string + item
            
        if not self.rendered:
            self.rendered=1 
            self.string = self.string + self.end_tag

        
###################################################
#            SHORTHAND SUBCLASSES                 #
###################################################

### SELECT ###
class Option(Closed_Tag):
    """Don't use this class explicitly. Add only strings to the Select
    class."""  
    def __init__(self, value, selected=0):
        if selected:
            Closed_Tag.__init__(self, "OPTION", ["SELECTED"], [value])
        else:
            Closed_Tag.__init__(self, "OPTION", [], [value])

class Select(Closed_Tag):
    """Shorthand for a SELECT tag"""
    def __init__(self, name, default):
        self.selected = default
        Closed_Tag.__init__(self, "SELECT", ["NAME='"+name+"' "], [])

    def mk_str(self):
        for item in self.content:
            if self.selected == item:
                self.string = self.string + str(Option(item, 1))
            else:
                self.string = self.string + str(Option(item))
        self.string = self.string + self.end_tag  

        
class Sticky_Select(Select):
    """Tries to retrieve value from Zope REQUEST object. Do not use
    Option objects with this class."""
    def __init__(self, REQUEST, name, default):
        Select.__init__(self, name, "")
        self.selected = REQUEST.get(name, default)

### TEXTAREA ###
class Textarea(Closed_Tag):
    """Shorthand for a Textarea tag"""
    def __init__(self, name, cols, rows, value="", attributes=[]):
        self.attributes = []
        self.attributes.insert(0, "NAME='" + name + "' ")
        self.attributes.insert(1, "COLS='" + cols + "' ")
        self.attributes.insert(2, "ROWS='" + rows + "' ")
        self.attributes = self.attributes + attributes                      
        Closed_Tag.__init__(self, "TEXTAREA", self.attributes,
                            [value])

class Sticky_Textarea(Textarea):
    """Tries to retrieve value from Zope REQUEST object."""
    def __init__(self, REQUEST, name, cols, rows, value="", attributes=[]):
        Textarea.__init__(self, name, cols, rows,
                          REQUEST.get(name, value), attributes)
    
### INPUT ###
class Input(Tag):
    """Shorthand for an INPUT tag"""
    def __init__(self, name, type="text", attributes=[], value=""):
        self.attributes = []
        self.attributes.insert(0, "NAME='"  + name + "' ")
        self.attributes.insert(1, "TYPE='"  + type + "' ")
        self.attributes.insert(2, "VALUE='" + value  + "' ")
        self.attributes = self.attributes + attributes
        Tag.__init__(self, "INPUT", self.attributes)

class Sticky_Input(Input):
    """Tries to retrieve value from Zope REQUEST object."""
    def __init__(self, REQUEST, name, type="text", attributes=[], value=""):
        Input.__init__(self, name, type, attributes, REQUEST.get(name, value))
        
### FORM ###
class Form(Closed_Tag):
    """Shorthand for a FORM tag."""
    def __init__(self, action, name="", content=[], method="post"):
        action = "ACTION='" + action + "' "
        name   = "NAME='"   + name   + "' "
        method = "METHOD='" + method + "' "
        Closed_Tag.__init__(self, "FORM", [action, name, method], content)

